//Подключаем необходимые библиотеки
//Для GPRG Shield

#include <GPRS_Shield_Arduino.h>
#include <SoftwareSerial.h>

//Для радиоприемника

#include <VirtualWire.h>

//Определяем пин, к которому подключено реле

#define RELAY 8

//Текущее состояние реле
 
bool stateRelay = false;

//Переменная для определения режима работы

int mode = 0;

//Переменная для ручного управления обогревателем

int m_temp = 0;

//Переменные для температуры
//Температура в данный момент

int curr_temp = 0;

//Температура срабатывания 

int trig_temp = 0;

//Нежелательная температура

int danger_temp = 36;

// создаём объект класса GPRS

GPRS gprs;

//Определяем номер, на который будем посылать сообщения
//Нужно ввести свой номер

#define PHONE_NUMBER "+XXXXXXXXXXX"

//Описываем сообщения для отправки

#define HELLO_MESSAGE "Hello from GPRS Shield! Please, set trigger temperature."
#define MODE_MESSAGE "Send mode's name."
#define READY_MESSAGE "I'm ready!"
#define INC_TRIG "Incorrect trigger value! You only can set - 20, 25 and 28."
#define INC_MODE "Incorrect mode value! You can only set - Auto and Manual"
#define DANGER_MESSAGE "Current temperature is bigger than 38C. Relay is off."
#define ERROR_SMS "Incorrect command. You can only ask for state, set trigger temperature or set mode"
#define HEATERON "OK, heater is on."
#define HEATEROFF "OK, heater is off."
#define AUTOMODE "OK, auto mode."
#define MANUALMODE "OK, manual mode."
#define OKTEMP "OK, trigger temperature changed"

String helloText = "Hello from GPRS Shield!";
String tempText = "Tempreature is ";
String termValue = String(curr_temp);
String degree = " C.";
String heaterOn = "Heater is On.";
String heaterOff = "Heater is Off.";
String message_to_send;
char char_message_to_send[60];

// текст сообщения
char message[160];
// номер, с которого пришло сообщение
char phone[16];
// дата отправки сообщения
char datetime[24];

void setup()
{
  //Определяем пины реле, светодиода, приемника
  
  pinMode(RELAY, OUTPUT);
  pinMode(13, OUTPUT);
  digitalWrite(RELAY, LOW);
  vw_set_ptt_inverted(true);
  vw_set_rx_pin(12);

  //Устанавливаем связь

  Serial.begin(9600);
  vw_setup(4000);
  vw_rx_start(); 

  Serial.println("Start");
  gprs.powerUpDown();
  
  while (!gprs.init()) 
   {
    delay(1000);
    Serial.println("Connecting");
   }
   
//Отправляем запрос температуры

 Serial.println("Connected");
 gprs.sendSMS(PHONE_NUMBER, HELLO_MESSAGE);
 Serial.println("Hello message sent");
 
 while(trig_temp == 0)
 {
  Serial.println("Waiting for trigger temperature");
    if (gprs.ifSMSNow()) 
   {
    gprs.readSMS(message, phone, datetime);
    sms_temp(message);
   } 
   delay(1000);
 }
 
 Serial.println("Trigger temperature assigned. Ask for mode name.");

 //Отправляем запрос режима
 
  gprs.sendSMS(PHONE_NUMBER, MODE_MESSAGE);
  
 while(mode == 0)
 {
   Serial.println("Waiting for mode");
   if (gprs.ifSMSNow()) 
   {
    gprs.readSMS(message, phone, datetime);
    sms_mode(message);   
   }
   delay(1000);
 }
  Serial.println("Mode assigned");

  gprs.sendSMS(PHONE_NUMBER, READY_MESSAGE);
}

void loop()
{
    //Ждем сообщения
    
    if (gprs.ifSMSNow()) 
   {
    gprs.readSMS(message, phone, datetime);
    Serial.println("Message");
    inc_sms(message);
   }
   //Получаем текущую температуру с термостата
   
   inc_temp();
   
   //Проверяем надобность включения реле
   
   relay();
}

//Включаем или выключаем реле

void relay()
{
  if(curr_temp < trig_temp && mode == 1)
  {
    digitalWrite(RELAY, HIGH);  
    stateRelay = true;
    delay(3000);
  }
   if(curr_temp >= trig_temp && mode == 1) 
  {
    digitalWrite(RELAY, LOW);
    stateRelay = false;
    delay(3000);
  }
  if(m_temp == 1 &&  mode == 2 && curr_temp < danger_temp)
  {
     digitalWrite(RELAY, HIGH);  
     stateRelay = true;
  }
  if(m_temp == 0 &&  mode == 2 || curr_temp >= danger_temp)
  {
     digitalWrite(RELAY, LOW);  
     stateRelay = false;
     if(curr_temp >= danger_temp)
     {
      Serial.println("DANGER!");
      gprs.sendSMS(PHONE_NUMBER, DANGER_MESSAGE);
      delay(60000);
     }
  }
}

//Проверяем входящее SMS

void inc_sms(char f_message[])
{
  if(strcmp(f_message, "State") == 0)
  {
    Serial.println("Calling for state function");
    state();
  }
  else if(strcmp(f_message, "HeaterOn") == 0)
  {
    m_temp = 1;
    gprs.sendSMS(PHONE_NUMBER, HEATERON);
  }
  else if(strcmp(f_message, "HeaterOff") == 0)
  {
    m_temp = 0;
    gprs.sendSMS(PHONE_NUMBER, HEATEROFF);
  }
  else if(strcmp(f_message, "Auto") == 0)
  {
    mode = 1;
    gprs.sendSMS(PHONE_NUMBER,AUTOMODE);
  }
  else if(strcmp(f_message, "Manual") == 0)
  {
    mode = 2;
    gprs.sendSMS(PHONE_NUMBER, MANUALMODE);
  }
  else if(strcmp(f_message, "20") == 0)
 {
  trig_temp = 20;
  gprs.sendSMS(PHONE_NUMBER, OKTEMP);
 }
 else if(strcmp(f_message, "25") == 0)
 {
  trig_temp = 25;
  gprs.sendSMS(PHONE_NUMBER, OKTEMP);
 }
 else if(strcmp(f_message, "28") == 0)
 {
  trig_temp = 28;
  gprs.sendSMS(PHONE_NUMBER, OKTEMP);
 }
  else 
  {
    Serial.println("Error");
    gprs.sendSMS(PHONE_NUMBER, ERROR_SMS);
    
  }
} 
 
//Устаналиваем режим

void sms_mode(char f_message[])
{
  if(strcmp(f_message, "Auto") == 0)
  {
    mode = 1;
  }
   else if(strcmp(f_message, "Manual") == 0)
  {
    mode = 2;
  }
  else
  {
    gprs.sendSMS(PHONE_NUMBER,INC_MODE);
  }
}
  
  /*
    Температура, которую нужно поддерживать.
    Я не буду делать так, чтобы можно было установить
    любую температуру срабатывания.
    Я выберу три - 20, 25 и 28.
    Вы можете выбирать режим на свое усмотрение.
  */
  void sms_temp(char f_message[])
{
 if(strcmp(f_message, "20") == 0)
 {
  trig_temp = 20;
 }
 else if(strcmp(f_message, "25") == 0)
 {
  trig_temp = 25;
 }
 else if(strcmp(f_message, "28") == 0)
 {
  trig_temp = 28;
 }
 else
 {
  gprs.sendSMS(PHONE_NUMBER,INC_TRIG);
 }
}

//Считываем температуру

void inc_temp()
{
  uint8_t buf[VW_MAX_MESSAGE_LEN]; // Буфер для сообщения
  uint8_t buflen = VW_MAX_MESSAGE_LEN; // Длина буфера
  char temp[3];
  
  if (vw_get_message(buf, &buflen)) // Если принято сообщение
  {
  Serial.println("Received:"); 
  digitalWrite(13, HIGH);
  for(int i = 0; i < buflen; i++)
  {
     temp[i] = buf[i];
  }
  curr_temp = atoi(temp);
  Serial.print("Temperature is ");
  Serial.print(temp);
  Serial.println(" C");
  digitalWrite(13, LOW);
 }
}

//Готовим и отправляем SMS со статусом

void state()
{
  if(stateRelay)
  {
    message_to_send = String(helloText + tempText + termValue + degree + heaterOn);
    message_to_send.toCharArray(char_message_to_send, 100);
    Serial.println("Sending");
    gprs.sendSMS(PHONE_NUMBER, char_message_to_send);
    Serial.println("Sent");
  }
  else if(!stateRelay)
  {
    message_to_send = String(helloText + tempText + termValue + degree + heaterOff);
    message_to_send.toCharArray(char_message_to_send, 100);
    Serial.println("Sending");
    gprs.sendSMS(PHONE_NUMBER, char_message_to_send);
    Serial.println("Sent");
  }
}

